@@ -27,11 +27,18 @@ use std::time::Duration;
 use futures::AsyncReadExt;
 use futures::TryStreamExt;
 use napi::bindgen_prelude::*;
+use opendal::options::{
+    DeleteOptions, ListOptions, ReadOptions, ReaderOptions, StatOptions, WriteOptions,
+};
 
 mod capability;
+mod options;
 
 #[napi]
-pub struct Operator(opendal::Operator);
+pub struct Operator {
+    async_op: opendal::Operator,
+    blocking_op: opendal::blocking::Operator,
+}
 
 #[napi]
 impl Operator {
@@ -49,25 +56,27 @@ impl Operator {
             .map_err(format_napi_error)?;
         let options = options.unwrap_or_default();
 
-        let mut op = opendal::Operator::via_iter(scheme, options).map_err(format_napi_error)?;
+        let async_op = opendal::Operator::via_iter(scheme, options).map_err(format_napi_error)?;
 
-        if !op.info().full_capability().blocking {
-            if let Ok(handle) = tokio::runtime::Handle::try_current() {
-                let _guard = handle.enter();
-                op = op.layer(
-                    opendal::layers::BlockingLayer::create()
-                        .expect("blocking layer must be created"),
-                );
-            }
-        }
+        let blocking_op = {
+            let handle = tokio::runtime::Handle::current();
+            let _guard = handle.enter();
 
-        Ok(Operator(op))
+            opendal::blocking::Operator::new(async_op.clone()).map_err(format_napi_error)?
+        };
+
+        Ok(Operator {
+            async_op,
+            blocking_op,
+        })
     }
 
     /// Get current operator(service)'s full capability.
     #[napi]
     pub fn capability(&self) -> Result<capability::Capability> {
-        Ok(capability::Capability::new(self.0.info().full_capability()))
+        Ok(capability::Capability::new(
+            self.async_op.info().full_capability(),
+        ))
     }
 
     /// Get current path's metadata **without cache** directly.
@@ -88,8 +97,17 @@ impl Operator {
     /// }
     /// ```
     #[napi]
-    pub async fn stat(&self, path: String) -> Result<Metadata> {
-        let meta = self.0.stat(&path).await.map_err(format_napi_error)?;
+    pub async fn stat(
+        &self,
+        path: String,
+        options: Option<options::StatOptions>,
+    ) -> Result<Metadata> {
+        let options = options.map_or_else(StatOptions::default, StatOptions::from);
+        let meta = self
+            .async_op
+            .stat_options(&path, options)
+            .await
+            .map_err(format_napi_error)?;
 
         Ok(Metadata(meta))
     }
@@ -104,8 +122,16 @@ impl Operator {
     /// }
     /// ```
     #[napi]
-    pub fn stat_sync(&self, path: String) -> Result<Metadata> {
-        let meta = self.0.blocking().stat(&path).map_err(format_napi_error)?;
+    pub fn stat_sync(
+        &self,
+        path: String,
+        options: Option<options::StatOptions>,
+    ) -> Result<Metadata> {
+        let options = options.map_or_else(StatOptions::default, StatOptions::from);
+        let meta = self
+            .blocking_op
+            .stat_options(&path, options)
+            .map_err(format_napi_error)?;
 
         Ok(Metadata(meta))
     }
@@ -120,7 +146,7 @@ impl Operator {
     /// ```
     #[napi]
     pub async fn check(&self) -> Result<()> {
-        self.0.check().await.map_err(format_napi_error)
+        self.async_op.check().await.map_err(format_napi_error)
     }
 
     /// Check the op synchronously.
@@ -131,7 +157,7 @@ impl Operator {
     /// ```
     #[napi]
     pub fn check_sync(&self) -> Result<()> {
-        self.0.blocking().check().map_err(format_napi_error)
+        self.blocking_op.check().map_err(format_napi_error)
     }
 
     /// Check if this path exists or not.
@@ -142,7 +168,7 @@ impl Operator {
     /// ```
     #[napi]
     pub async fn exists(&self, path: String) -> Result<bool> {
-        self.0.exists(&path).await.map_err(format_napi_error)
+        self.async_op.exists(&path).await.map_err(format_napi_error)
     }
 
     /// Check if this path exists or not synchronously.
@@ -153,7 +179,7 @@ impl Operator {
     /// ```
     #[napi]
     pub fn exists_sync(&self, path: String) -> Result<bool> {
-        self.0.blocking().exists(&path).map_err(format_napi_error)
+        self.blocking_op.exists(&path).map_err(format_napi_error)
     }
 
     /// Create dir with a given path.
@@ -164,7 +190,10 @@ impl Operator {
     /// ```
     #[napi]
     pub async fn create_dir(&self, path: String) -> Result<()> {
-        self.0.create_dir(&path).await.map_err(format_napi_error)
+        self.async_op
+            .create_dir(&path)
+            .await
+            .map_err(format_napi_error)
     }
 
     /// Create dir with a given path synchronously.
@@ -175,8 +204,7 @@ impl Operator {
     /// ```
     #[napi]
     pub fn create_dir_sync(&self, path: String) -> Result<()> {
-        self.0
-            .blocking()
+        self.blocking_op
             .create_dir(&path)
             .map_err(format_napi_error)
     }
@@ -188,10 +216,15 @@ impl Operator {
     /// const buf = await op.read("path/to/file");
     /// ```
     #[napi]
-    pub async fn read(&self, path: String) -> Result<Buffer> {
+    pub async fn read(
+        &self,
+        path: String,
+        options: Option<options::ReadOptions>,
+    ) -> Result<Buffer> {
+        let options = options.map_or_else(ReadOptions::default, ReadOptions::from);
         let res = self
-            .0
-            .read(&path)
+            .async_op
+            .read_options(&path, options)
             .await
             .map_err(format_napi_error)?
             .to_vec();
@@ -202,11 +235,20 @@ impl Operator {
     ///
     /// It could be used to read large file in a streaming way.
     #[napi]
-    pub async fn reader(&self, path: String) -> Result<Reader> {
-        let r = self.0.reader(&path).await.map_err(format_napi_error)?;
+    pub async fn reader(
+        &self,
+        path: String,
+        options: Option<options::ReaderOptions>,
+    ) -> Result<Reader> {
+        let options = options.map_or_else(ReaderOptions::default, ReaderOptions::from);
+        let r = self
+            .async_op
+            .reader_options(&path, options)
+            .await
+            .map_err(format_napi_error)?;
         Ok(Reader {
             inner: r
-                .into_futures_async_read(..)
+                .into_futures_async_read(std::ops::RangeFull)
                 .await
                 .map_err(format_napi_error)?,
         })
@@ -219,11 +261,11 @@ impl Operator {
     /// const buf = op.readSync("path/to/file");
     /// ```
     #[napi]
-    pub fn read_sync(&self, path: String) -> Result<Buffer> {
+    pub fn read_sync(&self, path: String, options: Option<options::ReadOptions>) -> Result<Buffer> {
+        let options = options.map_or_else(ReadOptions::default, ReadOptions::from);
         let res = self
-            .0
-            .blocking()
-            .read(&path)
+            .blocking_op
+            .read_options(&path, options)
             .map_err(format_napi_error)?
             .to_vec();
         Ok(res.into())
@@ -233,8 +275,16 @@ impl Operator {
     ///
     /// It could be used to read large file in a streaming way.
     #[napi]
-    pub fn reader_sync(&self, path: String) -> Result<BlockingReader> {
-        let r = self.0.blocking().reader(&path).map_err(format_napi_error)?;
+    pub fn reader_sync(
+        &self,
+        path: String,
+        options: Option<options::ReaderOptions>,
+    ) -> Result<BlockingReader> {
+        let options = options.map_or_else(ReaderOptions::default, ReaderOptions::from);
+        let r = self
+            .blocking_op
+            .reader_options(&path, options)
+            .map_err(format_napi_error)?;
         Ok(BlockingReader {
             inner: r.into_std_read(..).map_err(format_napi_error)?,
         })
@@ -256,31 +306,19 @@ impl Operator {
         &self,
         path: String,
         content: Either<Buffer, String>,
-        options: Option<WriteOptions>,
-    ) -> Result<()> {
+        options: Option<options::WriteOptions>,
+    ) -> Result<Metadata> {
         let c = match content {
             Either::A(buf) => buf.as_ref().to_owned(),
             Either::B(s) => s.into_bytes(),
         };
-        let mut writer = self.0.write_with(&path, c);
-        if let Some(options) = options {
-            if let Some(append) = options.append {
-                writer = writer.append(append);
-            }
-            if let Some(chunk) = options.chunk {
-                writer = writer.chunk(chunk.get_u64().1 as usize);
-            }
-            if let Some(ref content_type) = options.content_type {
-                writer = writer.content_type(content_type);
-            }
-            if let Some(ref content_disposition) = options.content_disposition {
-                writer = writer.content_disposition(content_disposition);
-            }
-            if let Some(ref cache_control) = options.cache_control {
-                writer = writer.cache_control(cache_control);
-            }
-        }
-        writer.await.map(|_| ()).map_err(format_napi_error)
+        let options = options.map_or_else(WriteOptions::default, WriteOptions::from);
+        let metadata = self
+            .async_op
+            .write_options(&path, c, options)
+            .await
+            .map_err(format_napi_error)?;
+        Ok(Metadata(metadata))
     }
 
     //noinspection DuplicatedCode
@@ -288,27 +326,18 @@ impl Operator {
     ///
     /// It could be used to write large file in a streaming way.
     #[napi]
-    pub async fn writer(&self, path: String, options: Option<WriterOptions>) -> Result<Writer> {
-        let mut writer = self.0.writer_with(&path);
-        if let Some(options) = options {
-            if let Some(append) = options.append {
-                writer = writer.append(append);
-            }
-            if let Some(chunk) = options.chunk {
-                writer = writer.chunk(chunk.get_u64().1 as usize);
-            }
-            if let Some(ref content_type) = options.content_type {
-                writer = writer.content_type(content_type);
-            }
-            if let Some(ref content_disposition) = options.content_disposition {
-                writer = writer.content_disposition(content_disposition);
-            }
-            if let Some(ref cache_control) = options.cache_control {
-                writer = writer.cache_control(cache_control);
-            }
-        }
-        let w = writer.await.map_err(format_napi_error)?;
-        Ok(Writer(w))
+    pub async fn writer(
+        &self,
+        path: String,
+        options: Option<options::WriteOptions>,
+    ) -> Result<Writer> {
+        let options = options.unwrap_or_default();
+        let writer = self
+            .async_op
+            .writer_options(&path, options.into())
+            .await
+            .map_err(format_napi_error)?;
+        Ok(Writer(writer))
     }
 
     /// Write multiple bytes into a path synchronously.
@@ -318,28 +347,14 @@ impl Operator {
     pub fn writer_sync(
         &self,
         path: String,
-        options: Option<WriterOptions>,
+        options: Option<options::WriteOptions>,
     ) -> Result<BlockingWriter> {
-        let mut writer = self.0.blocking().writer_with(&path);
-        if let Some(options) = options {
-            if let Some(append) = options.append {
-                writer = writer.append(append);
-            }
-            if let Some(chunk) = options.chunk {
-                writer = writer.chunk(chunk.get_u64().1 as usize);
-            }
-            if let Some(ref content_type) = options.content_type {
-                writer = writer.content_type(content_type);
-            }
-            if let Some(ref content_disposition) = options.content_disposition {
-                writer = writer.content_disposition(content_disposition);
-            }
-            if let Some(ref cache_control) = options.cache_control {
-                writer = writer.cache_control(cache_control);
-            }
-        }
-        let w = writer.call().map_err(format_napi_error)?;
-        Ok(BlockingWriter(w))
+        let options = options.unwrap_or_default();
+        let writer = self
+            .blocking_op
+            .writer_options(&path, options.into())
+            .map_err(format_napi_error)?;
+        Ok(BlockingWriter(writer))
     }
 
     //noinspection DuplicatedCode
@@ -358,31 +373,18 @@ impl Operator {
         &self,
         path: String,
         content: Either<Buffer, String>,
-        options: Option<WriteOptions>,
-    ) -> Result<()> {
+        options: Option<options::WriteOptions>,
+    ) -> Result<Metadata> {
         let c = match content {
             Either::A(buf) => buf.as_ref().to_owned(),
             Either::B(s) => s.into_bytes(),
         };
-        let mut writer = self.0.blocking().write_with(&path, c);
-        if let Some(options) = options {
-            if let Some(append) = options.append {
-                writer = writer.append(append);
-            }
-            if let Some(chunk) = options.chunk {
-                writer = writer.chunk(chunk.get_u64().1 as usize);
-            }
-            if let Some(ref content_type) = options.content_type {
-                writer = writer.content_type(content_type);
-            }
-            if let Some(ref content_disposition) = options.content_disposition {
-                writer = writer.content_disposition(content_disposition);
-            }
-            if let Some(ref cache_control) = options.cache_control {
-                writer = writer.cache_control(cache_control);
-            }
-        }
-        writer.call().map(|_| ()).map_err(format_napi_error)
+        let options = options.map_or_else(WriteOptions::default, WriteOptions::from);
+        let metadata = self
+            .blocking_op
+            .write_options(&path, c, options)
+            .map_err(format_napi_error)?;
+        Ok(Metadata(metadata))
     }
 
     /// Copy file according to given `from` and `to` path.
@@ -393,7 +395,10 @@ impl Operator {
     /// ```
     #[napi]
     pub async fn copy(&self, from: String, to: String) -> Result<()> {
-        self.0.copy(&from, &to).await.map_err(format_napi_error)
+        self.async_op
+            .copy(&from, &to)
+            .await
+            .map_err(format_napi_error)
     }
 
     /// Copy file according to given `from` and `to` path synchronously.
@@ -404,10 +409,7 @@ impl Operator {
     /// ```
     #[napi]
     pub fn copy_sync(&self, from: String, to: String) -> Result<()> {
-        self.0
-            .blocking()
-            .copy(&from, &to)
-            .map_err(format_napi_error)
+        self.blocking_op.copy(&from, &to).map_err(format_napi_error)
     }
 
     /// Rename file according to given `from` and `to` path.
@@ -420,7 +422,10 @@ impl Operator {
     /// ```
     #[napi]
     pub async fn rename(&self, from: String, to: String) -> Result<()> {
-        self.0.rename(&from, &to).await.map_err(format_napi_error)
+        self.async_op
+            .rename(&from, &to)
+            .await
+            .map_err(format_napi_error)
     }
 
     /// Rename file according to given `from` and `to` path synchronously.
@@ -433,8 +438,7 @@ impl Operator {
     /// ```
     #[napi]
     pub fn rename_sync(&self, from: String, to: String) -> Result<()> {
-        self.0
-            .blocking()
+        self.blocking_op
             .rename(&from, &to)
             .map_err(format_napi_error)
     }
@@ -449,8 +453,16 @@ impl Operator {
     /// await op.delete("test");
     /// ```
     #[napi]
-    pub async fn delete(&self, path: String) -> Result<()> {
-        self.0.delete(&path).await.map_err(format_napi_error)
+    pub async fn delete(
+        &self,
+        path: String,
+        options: Option<options::DeleteOptions>,
+    ) -> Result<()> {
+        let options = options.map_or_else(DeleteOptions::default, DeleteOptions::from);
+        self.async_op
+            .delete_options(&path, options)
+            .await
+            .map_err(format_napi_error)
     }
 
     /// Delete the given path synchronously.
@@ -460,8 +472,11 @@ impl Operator {
     /// op.deleteSync("test");
     /// ```
     #[napi]
-    pub fn delete_sync(&self, path: String) -> Result<()> {
-        self.0.blocking().delete(&path).map_err(format_napi_error)
+    pub fn delete_sync(&self, path: String, options: Option<options::DeleteOptions>) -> Result<()> {
+        let options = options.map_or_else(DeleteOptions::default, DeleteOptions::from);
+        self.blocking_op
+            .delete_options(&path, options)
+            .map_err(format_napi_error)
     }
 
     /// Remove given paths.
@@ -475,7 +490,26 @@ impl Operator {
     /// ```
     #[napi]
     pub async fn remove(&self, paths: Vec<String>) -> Result<()> {
-        self.0.delete_iter(paths).await.map_err(format_napi_error)
+        self.async_op
+            .delete_iter(paths)
+            .await
+            .map_err(format_napi_error)
+    }
+
+    /// Remove given paths.
+    ///
+    /// ### Notes
+    /// If underlying services support delete in batch, we will use batch delete instead.
+    ///
+    /// ### Examples
+    /// ```javascript
+    /// op.removeSync(["abc", "def"]);
+    /// ```
+    #[napi]
+    pub fn remove_sync(&self, paths: Vec<String>) -> Result<()> {
+        self.blocking_op
+            .delete_iter(paths)
+            .map_err(format_napi_error)
     }
 
     /// Remove the path and all nested dirs and files recursively.
@@ -489,7 +523,26 @@ impl Operator {
     /// ```
     #[napi]
     pub async fn remove_all(&self, path: String) -> Result<()> {
-        self.0.remove_all(&path).await.map_err(format_napi_error)
+        self.async_op
+            .remove_all(&path)
+            .await
+            .map_err(format_napi_error)
+    }
+
+    /// Remove the path and all nested dirs and files recursively.
+    ///
+    /// ### Notes
+    /// If underlying services support delete in batch, we will use batch delete instead.
+    ///
+    /// ### Examples
+    /// ```javascript
+    /// op.removeAllSync("path/to/dir/");
+    /// ```
+    #[napi]
+    pub fn remove_all_sync(&self, path: String) -> Result<()> {
+        self.blocking_op
+            .remove_all(&path)
+            .map_err(format_napi_error)
     }
 
     /// List the given path.
@@ -524,22 +577,19 @@ impl Operator {
     /// }
     /// ```
     #[napi]
-    pub async fn list(&self, path: String, options: Option<ListOptions>) -> Result<Vec<Entry>> {
-        let mut l = self.0.list_with(&path);
-        if let Some(options) = options {
-            if let Some(limit) = options.limit {
-                l = l.limit(limit as usize);
-            }
-            if let Some(recursive) = options.recursive {
-                l = l.recursive(recursive);
-            }
-        }
+    pub async fn list(
+        &self,
+        path: String,
+        options: Option<options::ListOptions>,
+    ) -> Result<Vec<Entry>> {
+        let options = options.map_or(ListOptions::default(), ListOptions::from);
+        let l = self
+            .async_op
+            .list_options(&path, options)
+            .await
+            .map_err(format_napi_error)?;
 
-        Ok(l.await
-            .map_err(format_napi_error)?
-            .iter()
-            .map(|e| Entry(e.to_owned()))
-            .collect())
+        Ok(l.into_iter().map(Entry).collect())
     }
 
     /// List the given path synchronously.
@@ -567,29 +617,26 @@ impl Operator {
     /// ```javascript
     /// const list = op.listSync("path/to/dir/", { recursive: true });
     /// for (let entry of list) {
-    ///   let meta = op.statSync(entry.path);
+    ///   let path = entry.path();
+    ///   let meta = entry.metadata();
     ///   if (meta.isFile) {
     ///     // do something
     ///   }
     /// }
     /// ```
     #[napi]
-    pub fn list_sync(&self, path: String, options: Option<ListOptions>) -> Result<Vec<Entry>> {
-        let mut l = self.0.blocking().list_with(&path);
-        if let Some(options) = options {
-            if let Some(limit) = options.limit {
-                l = l.limit(limit as usize);
-            }
-            if let Some(recursive) = options.recursive {
-                l = l.recursive(recursive);
-            }
-        }
+    pub fn list_sync(
+        &self,
+        path: String,
+        options: Option<options::ListOptions>,
+    ) -> Result<Vec<Entry>> {
+        let options = options.map_or(ListOptions::default(), ListOptions::from);
+        let l = self
+            .blocking_op
+            .list_options(&path, options)
+            .map_err(format_napi_error)?;
 
-        Ok(l.call()
-            .map_err(format_napi_error)?
-            .iter()
-            .map(|e| Entry(e.to_owned()))
-            .collect())
+        Ok(l.into_iter().map(Entry).collect())
     }
 
     /// Get a presigned request for read.
@@ -608,7 +655,7 @@ impl Operator {
     #[napi]
     pub async fn presign_read(&self, path: String, expires: u32) -> Result<PresignedRequest> {
         let res = self
-            .0
+            .async_op
             .presign_read(&path, Duration::from_secs(expires as u64))
             .await
             .map_err(format_napi_error)?;
@@ -631,7 +678,7 @@ impl Operator {
     #[napi]
     pub async fn presign_write(&self, path: String, expires: u32) -> Result<PresignedRequest> {
         let res = self
-            .0
+            .async_op
             .presign_write(&path, Duration::from_secs(expires as u64))
             .await
             .map_err(format_napi_error)?;
@@ -654,7 +701,7 @@ impl Operator {
     #[napi]
     pub async fn presign_stat(&self, path: String, expires: u32) -> Result<PresignedRequest> {
         let res = self
-            .0
+            .async_op
             .presign_stat(&path, Duration::from_secs(expires as u64))
             .await
             .map_err(format_napi_error)?;
@@ -673,6 +720,32 @@ impl Entry {
     pub fn path(&self) -> String {
         self.0.path().to_string()
     }
+
+    /// Return the metadata of this entry.
+    #[napi]
+    pub fn metadata(&self) -> Metadata {
+        Metadata(self.0.metadata().clone())
+    }
+}
+
+#[napi]
+pub enum EntryMode {
+    /// FILE means the path has data to read.
+    FILE,
+    /// DIR means the path can be listed.
+    DIR,
+    /// Unknown means we don't know what we can do on this path.
+    Unknown,
+}
+
+impl From<opendal::EntryMode> for EntryMode {
+    fn from(mode: opendal::EntryMode) -> Self {
+        match mode {
+            opendal::EntryMode::FILE => EntryMode::FILE,
+            opendal::EntryMode::DIR => EntryMode::DIR,
+            opendal::EntryMode::Unknown => EntryMode::Unknown,
+        }
+    }
 }
 
 /// Metadata carries all metadata associated with a path.
@@ -693,6 +766,19 @@ impl Metadata {
         self.0.is_file()
     }
 
+    /// This function returns `true` if the file represented by this metadata has been marked for
+    /// deletion or has been permanently deleted.
+    #[napi]
+    pub fn is_deleted(&self) -> bool {
+        self.0.is_deleted()
+    }
+
+    /// Cache-Control of this object.
+    #[napi(getter)]
+    pub fn cache_control(&self) -> Option<String> {
+        self.0.cache_control().map(|s| s.to_string())
+    }
+
     /// Content-Disposition of this object
     #[napi(getter)]
     pub fn content_disposition(&self) -> Option<String> {
@@ -705,6 +791,12 @@ impl Metadata {
         self.0.content_length().into()
     }
 
+    /// Content Encoding of this object
+    #[napi(getter)]
+    pub fn content_encoding(&self) -> Option<String> {
+        self.0.content_encoding().map(|s| s.to_string())
+    }
+
     /// Content MD5 of this object.
     #[napi(getter)]
     pub fn content_md5(&self) -> Option<String> {
@@ -717,6 +809,12 @@ impl Metadata {
         self.0.content_type().map(|s| s.to_string())
     }
 
+    /// User Metadata of this object.
+    #[napi(getter)]
+    pub fn user_metadata(&self) -> Option<HashMap<String, String>> {
+        self.0.user_metadata().cloned()
+    }
+
     /// ETag of this object.
     #[napi(getter)]
     pub fn etag(&self) -> Option<String> {
@@ -730,19 +828,25 @@ impl Metadata {
     pub fn last_modified(&self) -> Option<String> {
         self.0.last_modified().map(|ta| ta.to_rfc3339())
     }
-}
 
-#[napi(object)]
-pub struct ListOptions {
-    pub limit: Option<u32>,
-    pub recursive: Option<bool>,
+    /// mode represent this entry's mode.
+    #[napi(getter)]
+    pub fn mode(&self) -> Option<EntryMode> {
+        Some(self.0.mode().into())
+    }
+
+    /// Retrieves the `version` of the file, if available.
+    #[napi(getter)]
+    pub fn version(&self) -> Option<String> {
+        self.0.version().map(|v| v.to_string())
+    }
 }
 
 /// BlockingReader is designed to read data from a given path in a blocking
 /// manner.
 #[napi]
 pub struct BlockingReader {
-    inner: opendal::StdReader,
+    inner: opendal::blocking::StdReader,
 }
 
 #[napi]
@@ -780,7 +884,7 @@ impl Reader {
 /// BlockingWriter is designed to write data into a given path in a blocking
 /// manner.
 #[napi]
-pub struct BlockingWriter(opendal::BlockingWriter);
+pub struct BlockingWriter(opendal::blocking::Writer);
 
 #[napi]
 impl BlockingWriter {
@@ -870,70 +974,6 @@ impl Writer {
     }
 }
 
-#[napi(object)]
-#[derive(Default)]
-pub struct WriteOptions {
-    /// Append bytes into a path.
-    ///
-    /// ### Notes
-    ///
-    /// - It always appends content to the end of the file.
-    /// - It will create file if the path does not exist.
-    pub append: Option<bool>,
-
-    /// Set the chunk of op.
-    ///
-    /// If chunk is set, the data will be chunked by the underlying writer.
-    ///
-    /// ## NOTE
-    ///
-    /// A service could have their own minimum chunk size while perform write
-    /// operations like multipart uploads. So the chunk size may be larger than
-    /// the given buffer size.
-    pub chunk: Option<BigInt>,
-
-    /// Set the [Content-Type](https://developer.mozilla.org/en-US/docs/Web/HTTP/Headers/Content-Type) of op.
-    pub content_type: Option<String>,
-
-    /// Set the [Content-Disposition](https://developer.mozilla.org/en-US/docs/Web/HTTP/Headers/Content-Disposition) of op.
-    pub content_disposition: Option<String>,
-
-    /// Set the [Cache-Control](https://developer.mozilla.org/en-US/docs/Web/HTTP/Headers/Cache-Control) of op.
-    pub cache_control: Option<String>,
-}
-
-#[napi(object)]
-#[derive(Default)]
-pub struct WriterOptions {
-    /// Append bytes into a path.
-    ///
-    /// ### Notes
-    ///
-    /// - It always appends content to the end of the file.
-    /// - It will create file if the path does not exist.
-    pub append: Option<bool>,
-
-    /// Set the chunk of op.
-    ///
-    /// If chunk is set, the data will be chunked by the underlying writer.
-    ///
-    /// ## NOTE
-    ///
-    /// A service could have their own minimum chunk size while perform write
-    /// operations like multipart uploads. So the chunk size may be larger than
-    /// the given buffer size.
-    pub chunk: Option<BigInt>,
-
-    /// Set the [Content-Type](https://developer.mozilla.org/en-US/docs/Web/HTTP/Headers/Content-Type) of op.
-    pub content_type: Option<String>,
-
-    /// Set the [Content-Disposition](https://developer.mozilla.org/en-US/docs/Web/HTTP/Headers/Content-Disposition) of op.
-    pub content_disposition: Option<String>,
-
-    /// Set the [Cache-Control](https://developer.mozilla.org/en-US/docs/Web/HTTP/Headers/Cache-Control) of op.
-    pub cache_control: Option<String>,
-}
-
 /// Lister is designed to list entries at a given path in an asynchronous
 /// manner.
 #[napi]
@@ -961,7 +1001,7 @@ impl Lister {
 /// BlockingLister is designed to list entries at a given path in a blocking
 /// manner.
 #[napi]
-pub struct BlockingLister(opendal::BlockingLister);
+pub struct BlockingLister(opendal::blocking::Lister);
 
 /// Method `next` can be confused for the standard trait method `std::iter::Iterator::next`.
 /// But in JavaScript, it is also customary to use the next method directly to get the next element.
@@ -1031,7 +1071,19 @@ impl Operator {
     /// Add a layer to this operator.
     #[napi]
     pub fn layer(&self, layer: External<Layer>) -> Result<Self> {
-        Ok(Self(layer.inner.layer(self.0.clone())))
+        let async_op = layer.inner.layer(self.async_op.clone());
+
+        let blocking_op = {
+            let handle = tokio::runtime::Handle::current();
+            let _guard = handle.enter();
+
+            opendal::blocking::Operator::new(async_op.clone()).map_err(format_napi_error)?
+        };
+
+        Ok(Self {
+            async_op,
+            blocking_op,
+        })
     }
 }
 
@@ -1159,5 +1211,5 @@ impl RetryLayer {
 ///
 /// FIXME: handle error correctly.
 fn format_napi_error(err: impl Display) -> Error {
-    Error::from_reason(format!("{}", err))
+    Error::from_reason(format!("{err}"))
 }
